import sys
import os
import argparse
import argcomplete
import json
import textwrap
from provisionpad.db.database import load_database
from provisionpad.runs.create_instance import create_instance
from provisionpad.runs.terminate_instance import terminate_instance
from provisionpad.runs.stop_instance import stop_instance
from provisionpad.runs.start_instance import start_instance
from provisionpad.runs.create_vpc import create_vpc
from provisionpad.runs.initiate import initiate
from provisionpad.runs.status import show_status
from provisionpad.runs.attach_volume import attach_volume
from provisionpad.aws.aws_sg import AWSsgFuncs

shut_down_time = 50

from argparse import RawTextHelpFormatter

class PPAD(object):

    def __init__(self):
        parser = argparse.ArgumentParser(
            description='A very simple command line tool to control'
                        'your remote cloud based workstation',
            usage='''%(prog)s <command> [<args>]

The following commands are available:
    initiate   :   Initiate your test environment; only use it once
    create     :   Create a new computing instance
    terminate  :   Terminate the already running instance
    stop       :   Stop the running instance
    start      :   Start an stopped instance
    stat       :   Get the information on the current workspace
    allowhttp  :   Opens http(s) egress permission for security group
    resolvesg  :   Back to default security group
''')
        parser.add_argument('command', help='Choose one of (initiate, create, terminate, stop, start, stat) to run')
        args = parser.parse_args(sys.argv[1:2])
        if not hasattr(self, args.command):
            print ('Unrecognized command')
            parser.print_help()
            exit(1)
        getattr(self, args.command)()

    @staticmethod
    def get_env_vars():
        home = os.path.expanduser("~")
        env_dir = os.path.join(home, '.provisionpad')
        env_var_path = os.path.join(env_dir, 'env_variable.json')
        with open(env_var_path, 'r') as f:
            env_vars = json.load(f)
            if env_vars['env_path'] != env_var_path:
                print ('something wrong')
                raise ('The environment variable seems to be a wrong one')
        env_vars = {str(key):str(val) for key, val in env_vars.items()}
        return env_vars

    def initiate(self):
        initiate()

    def create(self):

        env_vars = PPAD.get_env_vars()
        DB = load_database(env_vars['db_path'])

        parser = argparse.ArgumentParser(
            description='Create a new computing instance',
            usage='''propad create [option]s

If no name is provided an automatic name starting with box will be used.
Please note you can not use names starting with box.
If no instance type is provided the default t2.micro will be sued for aws
as the instance type qualifies for the free tier

''')

        parser.add_argument('name', nargs='?', help='Enter the name you want to use')
        parser.add_argument('-t', '--type', dest='type', type=str, default='t2.micro', help='''Enter the type of computing instance
                                                    If you are using AWS see the following link for further info:
                                                    https://aws.amazon.com/ec2/pricing/on-demand/
                                                    ''')

        args = parser.parse_args(sys.argv[2:])

        if not args.name:
            boxname = ''
        else:
            boxname = args.name

        boxtype = args.type

        create_instance(boxname, boxtype, shut_down_time, env_vars, DB)

    def terminate(self):

        env_vars = PPAD.get_env_vars()
        DB = load_database(env_vars['db_path'])

        parser = argparse.ArgumentParser(
            description='Create a new computing instance',
            usage='''propad terminate thename

for example:
    propad terminate box2
    the command above will terminate the box2 and the associated root volume

''')

        parser.add_argument('name', nargs='?', help='Enter the name of the box you want to terminate')

        args = parser.parse_args(sys.argv[2:])
        boxname = args.name
        if not boxname:
            raise NameError('You need to enter the name of the box')
        terminate_instance(boxname, env_vars, DB)

    def stop(self):

        env_vars = PPAD.get_env_vars()
        DB = load_database(env_vars['db_path'])

        parser = argparse.ArgumentParser(
            description='Create a new computing instance',
            usage='''propad stop thename

for example:
    propad top box2
    the command above will stop the box2 and the associated root volume

''')

        parser.add_argument('name', nargs='?', help='Enter the name of the box you want to terminate')

        args = parser.parse_args(sys.argv[2:])

        boxname = args.name
        if not boxname:
            raise NameError('You need to enter the name of the box')

        stop_instance(boxname, env_vars, DB)

    def start(self):

        env_vars = PPAD.get_env_vars()
        DB = load_database(env_vars['db_path'])

        parser = argparse.ArgumentParser(
            description='Create a new computing instance',
            usage='''propad stop [instancename]

for example:
    propad start box2
    the command above will restart the stopped box2

''')

        parser.add_argument('name', nargs='?', help='Enter the name of the box you want to terminate')

        args = parser.parse_args(sys.argv[2:])

        boxname = args.name
        if not boxname:
            raise NameError('You need to enter the name of the box')

        start_instance(boxname, env_vars, DB)


    def stat(self):
        '''
        Prints out the stat of the running and stopped instances
        '''

        self.resolvesg()
        env_vars = PPAD.get_env_vars()
        DB = load_database(env_vars['db_path'])
        show_status(env_vars, DB)

    def allowhttp(self):
        '''
        allow http access for the given box name
        '''

        env_vars = PPAD.get_env_vars()
        DB = load_database(env_vars['db_path'])

        region = env_vars['aws_region']
        access_key = env_vars['access_key']
        secret_key = env_vars['secret_key']

        awssgf = AWSsgFuncs(region, access_key, secret_key)
        awssgf.check_public_ip(env_vars, DB)
        vpcparams = DB[env_vars['vpc_name'] ]
        awssgf.revoke_sg_permissions_all(vpcparams['vpc_id'])
        awssgf.set_sg_sshonly_local_ip(vpcparams['sg_id'], DB['public_ip'])
        awssgf.set_sg_http_egress(vpcparams['sg_id'])

    def resolvesg(self):
        '''
        limit the access to ssh-only from local machine.
        If you change your location then you need to update this function
        '''

        env_vars = PPAD.get_env_vars()
        DB = load_database(env_vars['db_path'])

        region = env_vars['aws_region']
        access_key = env_vars['access_key']
        secret_key = env_vars['secret_key']

        awssgf = AWSsgFuncs(region, access_key, secret_key)
        awssgf.check_public_ip(env_vars, DB)
        vpcparams = DB[env_vars['vpc_name'] ]
        awssgf.revoke_sg_permissions_all(vpcparams['vpc_id'])
        awssgf.set_sg_sshonly_local_ip(vpcparams['sg_id'], DB['public_ip'])

    def vol(self):
        '''
        attach volume to instance
        '''

        env_vars = PPAD.get_env_vars()
        DB = load_database(env_vars['db_path'])

        parser = argparse.ArgumentParser(
            description='Attach a volume to running instance',
            usage='''propad vol [instancename]

for example:
    propad vol box2 -s 10 -t g2

''')

        parser.add_argument('name', nargs='?', help='Enter the name of the box you want to terminate')
        parser.add_argument("-s", "--volsize", type=int, dest="volumesize",
                            help="Enter the volume size:")
        parser.add_argument("-t", "--voltype", dest="volumetype", default="gp2",
                            help="Enter the volume type")
        args = parser.parse_args(sys.argv[2:])

        boxname = args.name
        if not boxname:
            raise NameError('You need to enter the name of the box')
        elif boxname not in DB['running_instances']:
            raise ValueError('You can only attach a volume to running instance')

        volsize = args.volumesize
        if not volsize:
            raise ValueError('You need to provide the volume size')

        voltype = args.volumetype

        attach_volume(boxname, voltype, volsize, env_vars, DB)

def main():
   PPAD()

if __name__ == '__main__':
    main()




import sys
import os
from provisionpad.aws.aws_vol import AWSvolFuncs
from provisionpad.db.database import load_database, save_database
from provisionpad.runs.create_instance import run_command

def attach_volume(boxname, volume_type, volume_size, env_vars, DB):

    region = env_vars['aws_region']
    access_key = env_vars['access_key']
    secret_key = env_vars['secret_key']
    awsf = AWSvolFuncs(region, access_key, secret_key)

    params = {}
    try:
        nametuple = DB['running_instances'][boxname]['sdrive_names'].pop(0)
        params['name'] = nametuple[0]
        params['mnt']  = nametuple[1]
    except:
        raise Exception('Maximum number of volumes you can attach reached')

    params['az']    = DB['running_instances'][boxname]['az']
    params['instance_id'] = DB['running_instances'][boxname]['id']
    params['size']  = volume_size
    params['vtype'] = volume_type

    params['vol_id'] = awsf.create_volume(params)

    DB['running_instances'][boxname]['sdrive'][params['name']] = params
    save_database(DB, env_vars['db_path'])

    out = run_command(['ssh', boxname, 'sudo', 'mkfs', '-t', 'xfs', params['mnt']])
    drive_name = 'Project/{0}'.format(len(DB['running_instances'][boxname]['sdrive']))
    out = run_command(['ssh', boxname, 'sudo', 'mkdir', '-p', drive_name])
    out = run_command(['ssh', boxname, "echo 'sudo mount {0} ~/{1}' >> ~/.bashrc".format(params['mnt'], drive_name)])





import os
import sys
from provisionpad.aws.aws_ec2 import AWSec2Funcs
from provisionpad.db.database import load_database, save_database
from provisionpad.helpers.namehelpers import vpc_name
from provisionpad.helpers.texthelpers import write_into_text



def start_instance(boxname, env_vars, DB):

    if boxname not in DB['stopped_instances']:
        print ('the box is not available check again:')
        sys.exit()


    print ('Waiting for confirmation from AWS')
    region = env_vars['aws_region']
    home_folder = env_vars['HOME']
    access_key = env_vars['access_key']
    secret_key = env_vars['secret_key']
    awsf = AWSec2Funcs(region, access_key, secret_key)

    my_ssh_key_path = env_vars['key_pair_path']
    ssh_key_name = env_vars['key_pair_name']

    id = DB['stopped_instances'][boxname]['id']
    DB['running_instances'][boxname] = awsf.start_ec2_instance(id)
    DB['running_instances'][boxname]['sdrive'] = DB['stopped_instances'][boxname]['sdrive']
    del(DB['stopped_instances'][boxname])
    # DB['available_names'].append(boxname)
    save_database(DB, env_vars['db_path'])
    write_into_text(boxname,
'''
Host {0}
    HostName {1}
    User ubuntu
    IdentityFile {2}
    ForwardAgent yes
    StrictHostKeyChecking no
'''.format(boxname, DB['running_instances'][boxname]['public_ip'], my_ssh_key_path),
os.path.join(home_folder,'.ssh/config'))

    print ('ec2 instance {0} started successfully'.format(boxname))



import psutil
import time
import os
from os.path import expanduser
home = expanduser("~")
total_idlef = os.path.join(home, '.provisionpad/data/total_idle.out')
delta_idlef = os.path.join(home, '.provisionpad/data/delta_idle.out')
idle_time = psutil.cpu_times().idle
the_time  = time.time()
with open(total_idlef, 'r') as f:
    data = f.read().strip().split()
str = '%f %f' % (idle_time, the_time)
if len(data) != 2:
    with open(total_idlef, 'w') as f:
        f.write(str)
    with open(delta_idlef, 'w') as f:
        f.write('0\n')
else:
    idle_diff = (idle_time-float(data[0]))/(the_time-float(data[1]))
    with open(total_idlef, 'w') as f:
        f.write(str)
    with open(delta_idlef, 'a+') as f:
        f.write("%f\n" % idle_diff )  
    with open(delta_idlef, 'r') as f:
        data = f.read().splitlines()
        if float(data[-1])>0.97 and abs(float(data[-1])-float(data[-2]))<0.002:
            os.system('sudo poweroff')



import re
import os
import sys

def create_dir_for_file(filepath):
    thedir = os.path.dirname(os.path.realpath(filepath))
    if not os.path.isdir(thedir):
        os.makedirs(thedir)
        print('dir did not exist created one')

def write_into_text(marker, text, filetowrite):
    create_dir_for_file(filetowrite)
    pattern = '### PROVISIONPAD {0}\n.*?\n### PROVISIONPAD'.format(marker)
    if not os.path.isfile(filetowrite):
        with open(filetowrite, 'w+') as f:
            f.write(
            '### PROVISIONPAD {0}\n{1}\n### PROVISIONPAD\n'.format(
                marker, text.strip())
            )
    else:
        with open(filetowrite, 'r') as f:
            textfile = f.read()
        with open(filetowrite, 'a+') as f:
            f.write(
            '\n### PROVISIONPAD {0}\n{1}\n### PROVISIONPAD\n'.format(
                marker, text.strip())
            )            

def delete_text_from_file(marker, filetodelete):
    create_dir_for_file(filetodelete)
    pattern = '### PROVISIONPAD {0}\n.*?\n### PROVISIONPAD'.format(marker)
    if not os.path.isfile(filetodelete):
        print ('the file does not exists. Will ignore it for now') 
    else:
        with open(filetodelete, 'r') as f:
            textfile = f.read()
            modified_text = re.sub(pattern, '', textfile, flags=re.DOTALL)
        with open(filetodelete, 'w') as f:
            f.write(modified_text)

def clean_propad_from_file(filetodelete):
    create_dir_for_file(filetodelete)
    pattern = '### PROVISIONPAD.*?\n### PROVISIONPAD'
    if not os.path.isfile(filetodelete):
        pass
    else:
        with open(filetodelete, 'r') as f:
            textfile = f.read()
            modified_text = re.sub(pattern, '', textfile, flags=re.DOTALL)
            texttowrite = ''
            for line in modified_text.split('\n'):
                if line != '':
                    texttowrite += line+'\n'

        with open(filetodelete, 'w') as f:
            f.write(texttowrite)
                 




import os
import sys
import pickle
from collections import deque

def initiate_db(dbpath):
    print('initiating the database')
    database = {'created_instances': 0}
    database['available_names'] = deque()
    database['running_instances'] = {}
    database['stopped_instances'] = {}
    pickle.dump( database, open( dbpath, 'wb' ), protocol=2 )
    return database

def load_database(dbpath):
    if os.path.isfile(dbpath):
        return pickle.load(open(dbpath, 'rb'))
    else: # used for initializing the database
        return initiate_db(dbpath)

def save_database(database, dbpath):
    pickle.dump( database, open( dbpath, 'wb' ), protocol=2 )



import os
import sys
import time
from collections import defaultdict
import json

import boto3
from botocore.exceptions import ClientError
from collections import namedtuple



class AWSec2Funcs:

    def __init__(self, region, access_key, secret_key):
        self.region     = region
        self.access_key = access_key
        self.secret_key = secret_key

        self.ec2 = boto3.resource('ec2', region_name=region,
                                aws_access_key_id=access_key,
                                aws_secret_access_key=secret_key)

        self.client = boto3.client('ec2', region_name=region,
                                    aws_access_key_id=access_key,
                                    aws_secret_access_key=secret_key)


    def create_vpc(self, thename, cidrblock1, cidrblock2):

        vpctuple = namedtuple(thename, ['sg_id', 'subnet_id', 'vpc_id'])
        vpctuple.vpc_id = -1
        vpctuple.sg_id = -1
        vpctuple.subnet_id = -1

        try:
            # create VPC
            vpc = self.ec2.create_vpc(CidrBlock=cidrblock1)
            vpc.create_tags(Tags=[{'Key': 'Name', 'Value': thename}])
            vpc.wait_until_available()

            vpctuple.vpc_id = vpc.id

            for rt in vpc.route_tables.all():
                rt.create_tags(Tags=[{'Key': 'Name', 'Value': thename}])
                routetable = rt

            # enable public dns hostname so that we can SSH into it later
            self.client.modify_vpc_attribute( VpcId = vpc.id ,
                                     EnableDnsSupport = { 'Value': True } )
            self.client.modify_vpc_attribute( VpcId = vpc.id ,
                                     EnableDnsHostnames = { 'Value': True } )

            # create an internet gateway and attach it to VPC
            internetgateway = self.ec2.create_internet_gateway()
            time.sleep(1)
            internetgateway.create_tags(Tags=[{'Key': 'Name', 'Value': thename}])
            vpc.attach_internet_gateway(InternetGatewayId=internetgateway.id)

            routetable.create_route(DestinationCidrBlock='0.0.0.0/0',
                                    GatewayId=internetgateway.id)

            # create subnet and associate it with route table
            subnet = self.ec2.create_subnet(CidrBlock=cidrblock2, VpcId=vpc.id)
            subnet.create_tags(Tags=[{'Key': 'Name', 'Value': thename}])
            routetable.associate_with_subnet(SubnetId=subnet.id)

            # Create a security group and allow SSH inbound rule through the VPC
            for sg in vpc.security_groups.all():
                if sg.group_name == 'default':
                    sg.create_tags(Tags=[{'Key': 'Name', 'Value': thename}])
                    securitygroup = sg

            for acl in vpc.network_acls.all():
                acl.create_tags(Tags=[{'Key': 'Name', 'Value': thename}])

            vpctuple.sg_id = securitygroup.id
            vpctuple.subnet_id = subnet.id

        except:
            pass

        return vpctuple


    def delete_vpc(self, vpcid):

        vpc = self.ec2.Vpc(vpcid)
        # Delete custom security groups
        for sg in vpc.security_groups.all():
            if sg.group_name != 'default':
                sg.revoke_ingress(IpPermissions=sg.ip_permissions)
                sg.delete()

        # detach and delete all gateways associated with the vpc
        for gw in vpc.internet_gateways.all():
            vpc.detach_internet_gateway(InternetGatewayId=gw.id)
            gw.delete()

        # delete subnet
        for subnet in vpc.subnets.all():
            subnet.delete()

        for rt in vpc.route_tables.all():
            try:
                rt.delete()
            except:
                pass

        self.client.delete_vpc(VpcId=vpcid)

    def get_instance_info(self,id):
        instances_info = self.client.describe_instances()['Reservations']
        rdict = {}
        for x in instances_info:
            for y in x['Instances']:
                if y['InstanceId']== id:
                    rdict['id'] = id
                    rdict['launch_time'] = y['LaunchTime']
                    rdict['type'] = y['InstanceType']
                    rdict['public_dns'] = y['PublicDnsName'] if 'PublicDnsName' in y else 'NA'
                    rdict['public_ip'] = y['PublicIpAddress'] if 'PublicIpAddress' in y else 'NA'
                    rdict['private_dns'] = y['PrivateDnsName']
                    rdict['private_ip'] = y['PrivateIpAddress']
                    rdict['az'] = y['Placement']['AvailabilityZone'] # az is not me it is availabilityzone :D
                    rdict['pdrive'] = y['BlockDeviceMappings'][0]['Ebs']['VolumeId']
                    break
        return rdict

    def instance_state(self, your_name):
        instance_info = self.client.describe_instances()['Reservations']
        data = {}
        for x in instance_info:
            for y in x['Instances']:
                if 'Tags' in y:
                    for tag in y['Tags']:
                        if tag['Key']=='Name' and your_name in tag['Value']:
                            ip = y['PublicIpAddress'] if 'PublicIpAddress' in y else 'NA'
                            data[y['InstanceId']] = [y['State']['Name'], ip]
                            break
        return data

    def create_ec2_instance(self, params):

        instances = self.ec2.create_instances(
                        ImageId=params['aws_ami'],
                        InstanceType=params['box_type'],
                        MaxCount=1,
                        MinCount=1,
                        NetworkInterfaces=[{
                            'SubnetId': params['vpc']['subnet_id'],
                            'Groups': [params['vpc']['sg_id'],],
                            'DeviceIndex': 0,
                            'AssociatePublicIpAddress': True,
                        }],
                        TagSpecifications=[
                            {
                                'ResourceType': 'instance',
                                'Tags': [
                                    {
                                        'Key': 'Name',
                                        'Value': params['name']
                                    },
                                ]
                            },
                        ],
                        IamInstanceProfile={
                            'Name': params['aws_iam_role']
                        },
                        KeyName=params['ssh_key_name'])

        instances[0].wait_until_running()
        return self.get_instance_info(instances[0].id)

    def terminate_ec2_instance(self, id):
        ids = [id,]
        self.ec2.instances.filter(InstanceIds=ids).terminate()

    def stop_ec2_instance(self, id):
        ids = [id,]
        self.ec2.instances.filter(InstanceIds=ids).stop()

    def start_ec2_instance(self, id):
        ids = [id,]
        self.ec2.instances.filter(InstanceIds=ids).start()
        self.ec2.Instance(id=id).wait_until_running()
        return self.get_instance_info(id)

    def create_key_pair(self, key_name):
        return self.ec2.create_key_pair(KeyName=key_name).key_material

    def check_key_pair(self, key_name):
        key_pairs = self.client.describe_key_pairs()['KeyPairs']
        for key in key_pairs:
            if key['KeyName'] == key_name:
                return True
        return False



import sys
import time
from provisionpad.aws.aws_ec2 import AWSec2Funcs

class AWSvolFuncs(AWSec2Funcs):

    def volume_waiter(self, id, state):
        tw = 0
        while True:
            if tw > 120:
                print ('it is taking too long to make this volume avaialable; volume waiter')
                sys.exit()
            info = self.client.describe_volumes()['Volumes']
            for x in info:
                if x['VolumeId'] == id:
                    if x['State'] == state:
                        return
            time.sleep(5)
            tw += 5

    def create_volume(self, params):
        volume = self.ec2.create_volume(
            AvailabilityZone=params['az'],
            Size=params['size'],
            VolumeType=params['vtype']   , #'standard'|'io1'|'gp2'|'sc1'|'st1',
            TagSpecifications=[
                {
                    'ResourceType': 'volume',
                    'Tags': [
                        {
                            'Key': 'Name',
                            'Value': params['name']
                        },
                    ]
                },
            ]
        )
        self.volume_waiter(volume.id, 'available')
        volume.attach_to_instance(
            Device=params['mnt'],
            InstanceId=params['instance_id'],
        )
        time.sleep(2)
        return volume.id

    def get_volume_info(self, id):
        volume_info = self.client.describe_volumes()
        print (volume_info['Volumes'][0]['Attachments'])
        print (volume_info['Volumes'][1])
        print (len(volume_info['Volumes']) )

    def delete_vol(self, id):
        self.client.detach_volume(
            VolumeId=id,
            Force=True
        )
        self.volume_waiter(id, 'available')
        self.client.delete_volume(
            VolumeId=id
        )


import os
import sys
import time
import subprocess
from provisionpad.aws.aws_ec2 import AWSec2Funcs
from  provisionpad.aws.aws_sg import AWSsgFuncs
from provisionpad.db.database import load_database, save_database
from provisionpad.helpers.namehelpers import vpc_name
from provisionpad.helpers.texthelpers import write_into_text
from provisionpad.helpers.namehelpers import get_box_name
from provisionpad.helpers.update_status import update_status
from provisionpad.runs.status import show_status


def run_command(cmd_list):
    try:
        proc = subprocess.call(cmd_list,
                               stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        return proc.returncode
    except:
        proc = subprocess.call(cmd_list,
                               stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        return proc

def create_instance(boxname, boxtype, shut_down_time, env_vars, DB):

    update_status(env_vars,DB)

    region = env_vars['aws_region']
    home_folder = env_vars['HOME']
    access_key = env_vars['access_key']
    secret_key = env_vars['secret_key']
    awsf = AWSec2Funcs(region, access_key, secret_key)

    my_ssh_key_path = env_vars['key_pair_path']
    ssh_key_name = env_vars['key_pair_name']

    if not boxname:
        boxname = get_box_name(DB, env_vars['db_path'])
    else:
        if boxname[:3] == 'box' or \
                boxname in DB['running_instances'] or \
                boxname in DB['stopped_instances']:
            print ("enter a better name. either exists or starts with box")
            sys.exit()

    params = {}
    params['ssh_key_name'] = ssh_key_name
    params['aws_ami'] = env_vars['aws_ami']
    params['aws_iam_role'] = env_vars['role_name']
    params['vpc'] = DB[env_vars['vpc_name'] ]
    params['box_type'] = boxtype
    params['name'] = env_vars['your_name']+boxname


    print ('Waiting for confirmation from AWS')

    DB['running_instances'][boxname] = awsf.create_ec2_instance(params)
    write_into_text(boxname,
'''
Host {0}
    HostName {1}
    User ubuntu
    IdentityFile {2}
    ForwardAgent yes
    StrictHostKeyChecking no
'''.format(boxname, DB['running_instances'][boxname]['public_ip'], my_ssh_key_path),
os.path.join(home_folder,'.ssh/config'))

    drivel = 'fgh'
    DB['running_instances'][boxname]['sdrive_names'] = [(params['name']+'VOL{0}'.format(i), '/dev/xvd{0}'.format(drivel[i])) for i in range(len(drivel))]
    DB['running_instances'][boxname]['sdrive'] = {}
    save_database(DB, env_vars['db_path'])

    tmp_dir = os.path.join(env_vars['env_dir'], 'tmp')
    tmp_tclock = os.path.join(tmp_dir,'tclock.py')
    tmp_cron = os.path.join(tmp_dir,'cron')

    if not os.path.isdir(tmp_dir):
        os.mkdir(tmp_dir)


    dir_path = os.path.dirname(os.path.realpath(__file__))
    tmp_tclock = os.path.join(dir_path,'scripts', 'tclock.py')


    # The following line is OS dependent if it changes from ubuntu to something else then
    # It will cause trouble for auto shutdown -- Amir
    with open(tmp_cron, 'wb') as f:
        towrite = '*/{0} * * * * python /home/ubuntu/.provisionpad/tclock.py\n'.format(shut_down_time)
        f.write(towrite.encode('UTF-8'))

    print ('Setting up EC2 instance')

    awssgf = AWSsgFuncs(region, access_key, secret_key)
    awssgf.check_public_ip(env_vars, DB)
    vpcparams = DB[env_vars['vpc_name'] ]
    awssgf.revoke_sg_permissions_all(vpcparams['vpc_id'])
    awssgf.set_sg_sshonly_local_ip(vpcparams['sg_id'], DB['public_ip'])
    awssgf.set_sg_http_egress(vpcparams['sg_id'])

    # some times it has problem ssh ing into instance
    # I have added this 30 seconds sleep to be safe
    # Is there a better way? -- Amir
    time.sleep(30)




    out = run_command(['ssh', boxname, 'pip', 'install', 'psutil'])
    if out != 0:
        raise Exception('Failed to install psutil on server')
    out = run_command(['ssh', boxname, 'mkdir', '-p', '.provisionpad/data'])
    if out != 0:
        raise Exception('Failed to create .provisionpad/data')
    out = run_command(['ssh', boxname, 'touch', '.provisionpad/data/total_idle.out'])
    if out != 0:
        raise Exception('Failed to create .provisionpad/data/total_idle.out')
    out = run_command(['scp', tmp_cron,
                       '{0}:~/.provisionpad/'.format(boxname)])
    if out != 0:
        raise Exception('Failed to copy the crontab file')
    out = run_command(['scp', tmp_tclock,
                       '{0}:~/.provisionpad/'.format(boxname)])
    if out != 0:
        raise Exception('Failed to copy the tmp_tclock')
    out = run_command(['ssh', boxname, 'crontab',
                               '/home/ubuntu/.provisionpad/cron'])
    if out != 0:
        raise Exception('schedule a cron job')
    os.remove(tmp_cron)

    awssgf.revoke_sg_permissions_all(vpcparams['vpc_id'])
    awssgf.set_sg_sshonly_local_ip(vpcparams['sg_id'], DB['public_ip'])

    print ('ec2 instance {} created successfully'.format(boxname))
    show_status(env_vars, DB)




import os
import sys
from provisionpad.aws.aws_ec2 import AWSec2Funcs
from provisionpad.aws.aws_vol import AWSvolFuncs
from provisionpad.db.database import load_database, save_database
from provisionpad.helpers.namehelpers import vpc_name
from provisionpad.helpers.texthelpers import delete_text_from_file
from provisionpad.runs.create_instance import run_command
import textwrap


def terminate_instance(boxname, env_vars, DB):

    if boxname in DB['stopped_instances']:
        raise ValueError(textwrap.dedent('''\
              Instance {name} is in stopped state and can not be terminated
                          To terminate the instance you need to first start it:
                          propad start {name}; then you can stop it
        '''.format(name=boxname)))

    region = env_vars['aws_region']
    home_folder = env_vars['HOME']
    access_key = env_vars['access_key']
    secret_key = env_vars['secret_key']
    awsf = AWSec2Funcs(region, access_key, secret_key)
    awsvolf = AWSvolFuncs(region, access_key, secret_key)
    for _, val in DB['running_instances'][boxname]['sdrive'].items():
        id = val['vol_id']
        print (val['vol_id'], val['name'])
        out = run_command(['ssh', boxname, 'umount', val['mnt']])
        awsvolf.delete_vol(id)


    id = DB['running_instances'][boxname]['id']
    awsf.terminate_ec2_instance(id)
    del(DB['running_instances'][boxname])
    if boxname[0:3] == 'box':
        DB['available_names'].append(boxname)
    save_database(DB, env_vars['db_path'])
    delete_text_from_file(boxname, os.path.join(home_folder,'.ssh/config'))



    print ('ec2 instance {0} terminated successfully'.format(boxname))



import platform
import sys
import re
import os
import json
import subprocess
import boto3
import textwrap
from builtins import input
from provisionpad.aws.aws_ec2 import AWSec2Funcs
from  provisionpad.aws.aws_sg import AWSsgFuncs
from provisionpad.aws.aws_iam import AWSiamFuncs
from provisionpad.aws.aws_sts import AWSstsFuncs
from provisionpad.db.database import load_database, save_database
from provisionpad.runs.create_vpc import create_vpc

def initiate():

    home = os.path.expanduser("~")
    env_dir = os.path.join(home, '.provisionpad')
    if not os.path.isdir(env_dir):
        os.mkdir(env_dir)
    dbpath = os.path.join(env_dir, 'database.p')
    DB = load_database(dbpath)

    home = os.path.expanduser("~")
    env_dir = os.path.join(home, '.provisionpad')
    if not os.path.isdir(env_dir):
        os.mkdir(env_dir)

    env_var_path = os.path.join(env_dir, 'env_variable.json')
    input_var_path = os.path.join(env_dir, 'input_variable.json')


    if not os.path.isfile(input_var_path):
        env_vars = {}
        ask_for_credentials = True
        print ('Initiating a new propad environment')
        print ('Searching for default AWS credentials...')
        session = boto3.Session()
        credentials = session.get_credentials()
        if credentials:
            use_default_cred = input(textwrap.dedent('''\
                                        Default AWS credentials found.
                                        Do you want to use them?(y/n)
                                        '''))
            if str(use_default_cred).strip()[0] == 'y':
                env_vars['access_key'] = credentials.access_key
                env_vars['secret_key'] = credentials.secret_key
                ask_for_credentials = False
        if ask_for_credentials:
            print ('  You can find aws access keys under user tab (top third from right)')
            print ('  My security credentials for the root info or under IAM users section')
            print ('  For more information please visit: https://docs.aws.amazon.com/general/latest/gr/aws-sec-cred-types.html')
            access_key = input('Enter AWS access key ID: ')
            env_vars['access_key'] = str(access_key).strip()
            if not env_vars['access_key']:
                print ('Invalid input')
                sys.exit()
            secret_key = input('Enter AWS secret access key: ')
            env_vars['secret_key'] = str(secret_key).strip()
            if not env_vars['secret_key']:
                print ('Invalid input')
                sys.exit()
        your_name  = input('Enter your name (optional, will be added as a tag to the instance): ')
        env_vars['your_name'] = re.sub('[^a-zA-Z0-9]', '', your_name).upper()
        if not env_vars['your_name']:
            print ('Invalid input')
            sys.exit()
        # env_vars['your_email'] = input('Enter your email (): ')
        print ('\n')
        print ('Note: AMI (Image) should be in the same defined AWS region.')
        env_vars['aws_region'] = input ('Enter AWS region (us-east-2): ')
        if not env_vars['aws_region']:
            env_vars['aws_region'] = 'us-east-2'
        env_vars['aws_ami'] = input ('Enter your AWS AMI. (Ubuntu 18): ')
        if not env_vars['aws_ami']:
            env_vars['aws_ami'] = 'ami-029f8374ffdc9a057'  #'ami-00df714b389c23925'

        with open(input_var_path, 'w') as f:
            json.dump(env_vars, f, indent=4)

    else:

        with open(input_var_path, 'r') as f:
            env_vars = json.load(f)

    env_vars['db_path'] = dbpath
    env_vars['env_path'] = env_var_path
    env_vars['env_dir'] = env_dir

    key_pair_name = 'ec2_keypair_{0}_{1}.pem'.format(env_vars['your_name'], env_vars['aws_region'])
    key_pair_path = os.path.join(env_dir, key_pair_name)

    env_vars['key_pair_name'] = key_pair_name
    env_vars['key_pair_path'] = key_pair_path

    env_vars['vpc_name'] = '{0}_VPC'.format(env_vars['your_name'])

    role_name   = [env_vars['your_name'] ]
    policies = ['S3FULL']
    role_name.extend(policies)
    role_names = ''.join(role_name)

    env_vars['policy'] = policies
    env_vars['role_name'] = role_names

    env_vars['HOME'] = home

    # get the public ip address of local machine
    DB['public_ip'] = AWSsgFuncs.get_ip_address()
    save_database(DB, env_vars['db_path'])

    create_vpc(env_vars, DB)

    awsec2f = AWSec2Funcs(env_vars['aws_region'], env_vars['access_key'], env_vars['secret_key'])
    awsstsf = AWSstsFuncs(env_vars['aws_region'], env_vars['access_key'], env_vars['secret_key'])
    awsiamf = AWSiamFuncs(env_vars['aws_region'], env_vars['access_key'], env_vars['secret_key'])

    if not os.path.isfile(env_vars['key_pair_path']):
        if not awsec2f.check_key_pair(env_vars['key_pair_name']):
            try:
                print ('creating key pair')
                with open(env_vars['key_pair_path'], 'w') as f:
                    key_pair = str(awsec2f.create_key_pair(key_pair_name))
                    print (key_pair)
                    f.write(key_pair)
                os.chmod(env_vars['key_pair_path'], 0o600)
            except:
                os.remove(env_vars['key_pair_path'])
                raise Exception('You do not have access to create key-pair check your permissions')
        else:
            raise Exception('we can find the public key but pem is not available')
    else:
        print ('the key pair exists')

    account_id = awsstsf.get_account_id()
    policy_attach = []
    for policy in env_vars['policy']:
        policy_arn = 'arn:aws:iam::{0}:policy/{1}'.format(account_id, policy )
        if not awsiamf.check_policy_exists(policy_arn):
            if policy == 'S3FULL':
                awsiamf.ec2_policy_access_full(policy)
                policy_attach.append(policy_arn)
            else:
                print ('the policy {0} not implemented yet'.format(policy))
        else:
            print ('the policy {0} exists'.format(policy))
            policy_attach.append(policy_arn)

    if not awsiamf.check_role_exists(env_vars['role_name']):
        print (awsiamf.create_role_for_ec2(env_vars['role_name']) )
        awsiamf.create_instance_profile(env_vars['role_name'])

    if awsiamf.check_role_exists(env_vars['role_name'], 1, 5):
        for policy in policy_attach:
            print ('attaching policy arn: {0}'.format(policy))
            awsiamf.attach_policy_to_role(env_vars['role_name'], policy)
            print ('policy attached')
    else:
        raise Exception(' was not able to find the role')


    with open(env_var_path, 'w') as f:
        json.dump(env_vars, f, indent=4)


import os
import sys
import boto3
from botocore.exceptions import ClientError
import json
import time
from provisionpad.aws.aws_sts import AWSstsFuncs

class AWSiamFuncs:

    def __init__(self, region, access_key, secret_key):
        self.region     = region
        self.access_key = access_key
        self.secret_key = secret_key 

        self.client = boto3.client('iam', region_name=region,
                                    aws_access_key_id=access_key,
                                    aws_secret_access_key=secret_key )

        self.iam = boto3.resource('iam', region_name=region,
                                    aws_access_key_id=access_key,
                                    aws_secret_access_key=secret_key )

        self.sts = AWSstsFuncs(region, access_key, secret_key)

    def ec2_policy_access_full(self, policy_name):
        account_id = self.sts.get_account_id()
        policy_arn = 'arn:aws:iam::{0}:policy/{1}'.format(account_id, policy_name)
        if self.check_policy_exists(policy_arn):
            return policy_arn
        else:
            policy_doc = {
                            "Version": "2012-10-17",
                            "Statement": [{"Effect": "Allow",
                                        "Action": "s3:*",
                                        "Resource": "*"
                                        }
                            ]
            }
            self.client.create_policy(
                PolicyName=policy_name,
                PolicyDocument=json.dumps(policy_doc),
                Description='grant full access to s3'
            )
            if not self.check_policy_exists(policy_arn, delay=2, maxattempts=3):
                raise Exception('It seems it was not able to create the policy check your AWS permissions')
            return policy_arn

    def check_policy_exists(self, policy_arn, delay=1, maxattempts=1):
        waiter = self.client.get_waiter('policy_exists')
        try:
            waiter.wait(
                PolicyArn=policy_arn,
                WaiterConfig={
                    'Delay': delay,
                    'MaxAttempts': maxattempts
                }
            )
        except: 
            return False
        return True

    def create_role_for_ec2(self, role_name):  
        account_id = self.sts.get_account_id()
        role_arn = 'arn:aws:iam::{0}:role/{1}'.format(account_id, role_name)
        base_policy = { "Version": "2012-10-17", 
                        "Statement": [ { "Effect": "Allow", 
                                         "Principal": { "Service": "ec2.amazonaws.com" }, 
                                         "Action": "sts:AssumeRole" } ] }  
        response = self.client.create_role(
            RoleName=role_name,
            AssumeRolePolicyDocument=json.dumps(base_policy),
            Description='ec2_policy',
        )
        return role_arn

    def attach_policy_to_role(self, role_name, policy_arn):
        role = self.iam.Role(role_name)
        response = role.attach_policy(PolicyArn=policy_arn)

    def check_role_exists(self, role_name, delay=1, maxattempts=1):
        try:
            waiter = self.client.get_waiter('role_exists')
            waiter.wait(
                RoleName=role_name,
                WaiterConfig={
                    'Delay': delay,
                    'MaxAttempts': maxattempts
                }
            )
        except:
            return False
        return True

    def create_instance_profile(self, name):
        instance_profile = self.iam.create_instance_profile(
            InstanceProfileName=name,
        )
        time.sleep(1)
        instance_profile.add_role(
            RoleName=name
        )






from __future__ import print_function

from colorclass import Color, Windows

from terminaltables import SingleTable
from provisionpad.helpers.update_status import update_status
import sys

class StatTable:

    def __init__(self, instance_status, color):
        self.color = color
        self.instance_status = instance_status

    def stat(self, DB):
        """Return table string to be printed."""
        table_data = [[Color('{'+self.color+'}Name{/'+self.color+'}'), 'Type', 'Volumes', 'SSH']]
        for ins, ins_val in DB[self.instance_status].items():
            volume = 'root;'
            for _, val in ins_val['sdrive'].items():
                volume += ' {0} GB;'.format(val['size'])
            table_data.append([Color('{'+self.color+'}'+ins+'{/'+self.color+'}'),
                                ins_val['type'], volume,'ssh {0}'.format(ins)])
        table_instance = SingleTable(table_data, self.instance_status)
        table_instance.inner_heading_row_border = True
        return table_instance.table

    def sstat(self, DB):
        """Return table string to be printed."""
        str_to_return  = '\n'+self.instance_status+'\n'
        str_to_return += ' ' *len(self.instance_status) + \
                   "{:<20} {:<20} {:<20} {:<20}\n".format('Name', 'Type', 'Volumes', 'SSH')
        for ins, ins_val in DB[self.instance_status].items():
            volume = 'root;'
            for _, val in ins_val['sdrive'].items():
                volume += ' {0} GB;'.format(val['size'])
            str_to_return += ' ' *len(self.instance_status) + \
                   "{:<20} {:<20} {:<20} {:<20}\n".format(ins, ins_val['type'], volume, 'ssh {0}\n'.format(ins))
        str_to_return += '\n'
        return str_to_return

def show_status(env_vars, DB):
    update_status(env_vars,DB)
    # print (DB)
    table_running = StatTable('running_instances', 'autogreen')
    table_stopped = StatTable('stopped_instances', 'autoyellow')
    if sys.platform == 'win32':
        print (table_running.sstat(DB))
        print (table_stopped.sstat(DB))
    else:
        print (table_running.stat(DB))
        print (table_stopped.stat(DB))













import boto3

class AWSstsFuncs:

    def __init__(self, region, access_key, secret_key):

        self.client = boto3.client('sts', region_name=region,
                                    aws_access_key_id=access_key,
                                    aws_secret_access_key=secret_key )

    def get_account_id(self):
        return self.client.get_caller_identity()["Account"]


import os
import sys
from provisionpad.aws.aws_ec2 import AWSec2Funcs
# from provisionpad.helpers.namehelpers import vpc_name
from provisionpad.db.database import load_database, save_database

def create_vpc(env_vars, DB):
    region = env_vars['aws_region']
    access_key = env_vars['access_key']
    secret_key = env_vars['secret_key']
    awsf = AWSec2Funcs(region, access_key, secret_key)
    thename = env_vars['vpc_name']
    filters = [{'Name':'tag:Name', 'Values':[thename]}]
    vpcs = list(awsf.ec2.vpcs.filter(Filters=filters))

    if len(vpcs)==0:
        pass
    elif len(vpcs)==1 and thename in DB:
        print ('The VPC with the name: {0} exists.'.format(thename))
        print ('will continue to use it')
        return
    elif thename not in DB:
        pass
    elif len(vpcs)>1:
        raise Exception('There are more than one VPCs with the name. Contact ...')
    else:
        raise Exception('Either VPC does not exists or DB has changed')

    cidrblock1 = '172.16.0.0/28'
    cidrblock2 = '172.16.0.0/28'

    vpc_params = awsf.create_vpc(thename, cidrblock1, cidrblock2)
    vpcid = vpc_params.vpc_id
    if type(vpc_params.vpc_id)==type(-1):
        raise Exception('Was not able to create VPC. Check your permissions')
    elif type(vpc_params.sg_id)==type('s') and type(vpc_params.vpc_id)==type(-1):
        awsf.delete_vpc(vpcid)
        raise Exception('Was able to create basic VPC but fails further down with VPC dependencies')


    print (vpc_params.sg_id, vpc_params.subnet_id, vpc_params.vpc_id)
    DB[thename] = {}
    DB[thename]['vpc_id']    = vpc_params.vpc_id
    DB[thename]['sg_id']     = vpc_params.sg_id
    DB[thename]['subnet_id'] = vpc_params.subnet_id
    save_database(DB, env_vars['db_path'])




import os
import sys
from provisionpad.db.database import save_database


def vpc_name():
    return os.environ['your_name'].replace(" ", "")+'_VPC'


def get_box_name(DB, dbpath):
    """
    gets redis variables names and returns the best name for the
    newly created instance
    """
    if len(DB['available_names']) > 0:
        dname = DB['available_names'].popleft()
        if dname in DB['running_instances'] or\
                dname in DB['stopped_instances'] :
            raise Exception('Was not able to find a proper name. Please report the bug')
        boxn = dname
    else:
        boxi = DB['created_instances'] + 1
        boxn = 'box{0}'.format(boxi)
        DB['created_instances'] += 1
        save_database(DB, dbpath)
    return boxn


import os
import sys
from provisionpad.aws.aws_ec2 import AWSec2Funcs
from provisionpad.db.database import load_database, save_database
from provisionpad.helpers.namehelpers import vpc_name
from provisionpad.helpers.texthelpers import delete_text_from_file

     

def stop_instance(boxname, env_vars, DB):

    region = env_vars['aws_region']
    home_folder = env_vars['HOME']
    access_key = env_vars['access_key']
    secret_key = env_vars['secret_key']
    awsf = AWSec2Funcs(region, access_key, secret_key)

    id = DB['running_instances'][boxname]['id']
    awsf.stop_ec2_instance(id)
    DB['stopped_instances'][boxname] = DB['running_instances'][boxname]
    del(DB['running_instances'][boxname])
    save_database(DB, env_vars['db_path'])
    delete_text_from_file(boxname, os.path.join(home_folder,'.ssh/config'))

    print ('ec2 instance {0} stopped successfully'.format(boxname))



from provisionpad.aws.aws_ec2 import AWSec2Funcs
from requests import get
from provisionpad.db.database import load_database, save_database


class AWSsgFuncs(AWSec2Funcs):

    @staticmethod
    def get_ip_address():
        '''
        Returns:
            Public IP address of the machine
        '''
        return get('https://api.ipify.org').text

    def check_public_ip(self, env_vars, DB):
        thepublicip = AWSsgFuncs.get_ip_address()
        if thepublicip != DB['public_ip']:
            print ('Your public_ip has changed from {0} to {1}'.format(DB['public_ip'], thepublicip))
            DB['public_ip'] = thepublicip
        save_database(DB, env_vars['db_path'])

    def set_sg_sshonly_local_ip(self, sgid, publicip):
        '''
        modifies the security group to only ssh ingress and
        no egress

        Parameters:
            sgid (str): the id of the security group

        Returns:
            -
        '''
        # ip = AWSsgFuncs.get_ip_address()
        myipaddress = '{0}/32'.format(publicip)
        securitygroup = self.ec2.SecurityGroup(sgid)
        # using json request format like set_sg_http_egress
        # was producing strange errors. Just used the following format --Amir
        securitygroup.authorize_ingress(CidrIp=myipaddress,
                                        IpProtocol='tcp',
                                        FromPort=22,
                                        ToPort=22)



    def set_sg_http_egress(self, sgid):
        '''
        modifies the security group to only ssh ingress and
        no egress

        Parameters:
            sgid (str): the id of the security group

        Returns:
            -
        '''
        securitygroup = self.ec2.SecurityGroup(sgid)
        securitygroup.authorize_egress(
            IpPermissions=[
                {'IpProtocol': 'tcp',
                'FromPort': 80,
                'ToPort': 80,
                'IpRanges':   [{'CidrIp': '0.0.0.0/0'}],
                'Ipv6Ranges': [{'CidrIpv6': '::/0'}]},
                {'IpProtocol': 'tcp',
                'FromPort': 443,
                'ToPort': 443,
                'IpRanges':   [{'CidrIp': '0.0.0.0/0'}],
                'Ipv6Ranges': [{'CidrIpv6': '::/0'}]},
            ]
        )


    def revoke_sg_permissions_all(self,vpcid):
        '''
        revokes all the egress and ingress security permissions

        Parameters:
            vpcid (str): the id of vpc to revoke permissions

        Returns:
            -
        '''
        clientsecgrps = self.client.describe_security_groups(Filters=[ { 'Name': 'vpc-id', 'Values': [vpcid] }])
        groups = clientsecgrps['SecurityGroups']

        for group in groups:
            if len(group['IpPermissions']) > 0:
                for permission in group['IpPermissions']:
                    self.client.revoke_security_group_ingress(GroupId=group['GroupId'],IpPermissions=[permission])
            if len(group['IpPermissionsEgress']) > 0:
                for permission in group['IpPermissionsEgress']:
                    self.client.revoke_security_group_egress(GroupId=group['GroupId'],IpPermissions=[permission])



import sys
import os

from os.path import expanduser
home = expanduser("~")

try:
    from setuptools import setup, find_packages
except ImportError:
    print("Please install setup tools.")
    sys.exit(1)

def read_file(file_name):
    '''
    Read a file and return stringls
    '''
    with open(file_name, 'r') as f:
        return f.read()

setup (
    name='provisionpad',
    version='0.0.3',
    description='The easiest way to create and connect to secure cloud instances',
    long_description=read_file('README.md'),
    author='Amir Zainali',
    author_email='provisionpad@gmail.com',
    url='https://www.provisionpad.com/',
    install_requires=['boto3',
                      'python-dateutil',
                      'future',
                      'argcomplete',
                      'colorclass',
                      'terminaltables',
                      'tzlocal',
                      'requests'],
    project_urls={
        'Source Code': 'https://github.com/provisionpad/provisionpad',
    },
    license='Apache-2.0',
    # python_requires='>=2.7, >=3.7',
    package_dir={'': 'src'},
    packages=find_packages('src'),
    classifiers=[
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
    ],
    entry_points={
        'console_scripts': [
            'propad = provisionpad.bin.propad:main',
        ]
    }
)





import os
import sys
from provisionpad.db.database import load_database, save_database
from provisionpad.aws.aws_ec2 import AWSec2Funcs
from provisionpad.helpers.texthelpers import delete_text_from_file, write_into_text
from provisionpad.helpers.texthelpers import clean_propad_from_file
from copy import deepcopy
from provisionpad.helpers.namehelpers import get_box_name

# 0 : pending # 16 : running # 32 : shutting-down
# 48 : terminated  # 64 : stopping # 80 : stopped

def update_status(env_vars, DB):

    region = env_vars['aws_region']
    home_folder = env_vars['HOME']
    access_key = env_vars['access_key']
    secret_key = env_vars['secret_key']
    awsec2f = AWSec2Funcs(region, access_key, secret_key)
    aws_inst_info = awsec2f.instance_state(env_vars['your_name'])
    aws_inst_info_d = deepcopy(aws_inst_info)

    clean_propad_from_file(os.path.join(home_folder,'.ssh/config'))

    for ins in aws_inst_info:
        if aws_inst_info[ins][0] == 'terminated':
            aws_inst_info_d.pop(ins)
        if not (aws_inst_info[ins][0] == 'stopped' or
                aws_inst_info[ins][0] == 'running' or
                aws_inst_info[ins][0] == 'terminated'):
            print ('Waiting to complete the transition. Try again a bit later.')
            sys.exit()

    aws_inst_info = aws_inst_info_d

    DBD = deepcopy(DB)
    for ins, ins_info in DBD['running_instances'].items():
        if ins_info['id'] not in aws_inst_info:
            print ('seems like the instance you have created ')
            print ('has been removed from the aws manually most likely')
            print ('removing it from the database')
            del(DB['running_instances'][ins])
            if ins[0:3] == 'box':
                DB['available_names'].append(ins)
            save_database(DB, env_vars['db_path'])
            delete_text_from_file(ins, os.path.join(home_folder,'.ssh/config'))
        elif ins_info['id'] in aws_inst_info and aws_inst_info[ins_info['id']][0]=='stopped':
            print ('seems like the instance has been stopped')
            print ('removing it from the running instances')
            DB['stopped_instances'][ins] = DB['running_instances'][ins]
            del(DB['running_instances'][ins])
            save_database(DB, env_vars['db_path'])
            delete_text_from_file(ins, os.path.join(home_folder,'.ssh/config'))
        else:
            write_into_text(ins,
'''
Host {0}
    HostName {1}
    User ubuntu
    IdentityFile {2}
    ForwardAgent yes
    StrictHostKeyChecking no
'''.format(ins, DB['running_instances'][ins]['public_ip'], env_vars['key_pair_path']),
os.path.join(home_folder,'.ssh/config'))
            print ('{0} is fine as expected'.format(ins))

    DBD = deepcopy(DB)
    for ins, ins_info in DBD['stopped_instances'].items():
        if ins_info['id'] not in aws_inst_info:
            print ('seems like the instance you have created ')
            print ('has been removed from the aws manually most likely')
            print ('removing it from the database')
            del(DB['stopped_instances'][ins])
            if ins[0:3] == 'box':
                DB['available_names'].append(ins)
            save_database(DB, env_vars['db_path'])
            delete_text_from_file(ins, os.path.join(home_folder,'.ssh/config'))
        elif ins_info['id'] in aws_inst_info and aws_inst_info[ins_info['id']][0]=='running':
            print ('seems like the instance has started manually')
            print ('moving from stopped to running')
            DB['running_instances'][ins] = DB['stopped_instances'][ins]
            DB['running_instances'][ins]['public_ip'] = aws_inst_info[ins_info['id']][1]
            del(DB['stopped_instances'][ins])
            save_database(DB, env_vars['db_path'])
            write_into_text(ins,
'''
Host {0}
    HostName {1}
    User ubuntu
    IdentityFile {2}
    ForwardAgent yes
    StrictHostKeyChecking no
'''.format(ins, DB['running_instances'][ins]['public_ip'], env_vars['key_pair_path']),
os.path.join(home_folder,'.ssh/config'))
        else:
            print ('{0} is fine as expected'.format(ins))

    ids_db = set([])
    for x in DB['running_instances']:
        ids_db.add(DB['running_instances'][x]['id'])
    for x in DB['stopped_instances']:
        ids_db.add(DB['stopped_instances'][x]['id'])

    to_create = set(aws_inst_info) - ids_db
    for x in to_create:
        ins = get_box_name(DB, env_vars['db_path'])
        thekeyname = 'stopped_instances'
        if aws_inst_info[x][0] == 'running':
            thekeyname = 'running_instances'
        boxname = get_box_name(DB, env_vars['db_path'])
        DB[thekeyname][boxname] = awsec2f.get_instance_info(x)
        write_into_text(ins,
'''
Host {0}
    HostName {1}
    User ubuntu
    IdentityFile {2}
    ForwardAgent yes
    StrictHostKeyChecking no
'''.format(ins, DB[thekeyname][boxname]['public_ip'], env_vars['key_pair_path']),
os.path.join(home_folder,'.ssh/config'))

    if len(ids_db - set(aws_inst_info))>0:
        raise Exception('this should not happen in status')


    save_database(DB, env_vars['db_path'])